﻿
-- TODO:
-- track data for login/out
-- report data for login logout
-- setup configurations on who to track / report
--

--FRIEND API'S:
--AddFriend("playerName") - Add a friend to your friend list. 
--AddOrRemoveFriend("playerName"[, "note"]) - Toggles a player's presence on your friends list. 
--GetFriendInfo(index) - Returns name, level, class, location, connected, status, and friend note of a friend. 
--SetFriendNotes(index, "note") - Sets the note text for a friend. 
--GetNumFriends() - Returns how many friends are on your friend list. 
--GetSelectedFriend() - Returns the index of the current selected friend. 
--RemoveFriend("name" or index) - Removes a friend from your friend list 
--SetSelectedFriend(index) - Update the current selected friend. 
--ShowFriends() - Request updated friends information from server. 
--UI ToggleFriendsFrame([tabNumber]) - Opens/closes the friends pane (possibly on a specific tab). 


FriendTracker = LibStub("AceAddon-3.0"):NewAddon("FriendTracker", "AceConsole-3.0", "AceBucket-3.0", "AceEvent-3.0")


function FriendTracker:OnInitialize()
	FriendTracker:Print("Friend Tracker Initialized")

	--initialize options
	local options = {
	    name = "FriendTracker",
	    handler = FriendTracker,
	    type = 'group',
	    args = {
            reset = {
                type = 'execute',
                name = 'Reset Database',
                desc = 'Resets Friend Tracker to first time install state.',
                func = 'ResetDatabase',
            },
            sf = {
                type = 'execute',
                name = 'scan friends',
                desc = 'Scan Friends',
                func = 'ScanFriends',
            },
            print = {
                type = 'execute',
                name = 'Print Queue',
                desc = 'Print the event queue to console.',
                func = 'PrintQueue',
            },
            clear = {
                type = 'execute',
                name = 'Clear Print Queue',
                desc = 'Clear the event queue.',
                func = 'ClearQueue',
            },
            show = {
                type = 'execute',
                name = 'Show Friend Tracker',
                desc = 'Displays the Friend Tracker UI',
                func = 'ShowFrame',
            },
        }
	}
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FriendTracker", options, {"friendtracker", "ft"})
	
    FriendTracker.AceGUI = LibStub("AceGUI-3.0")
    
	FriendTracker:InitDB()
end

function FriendTracker:InitDB()
	FriendTracker.db = LibStub("AceDB-3.0"):New("FriendTrackerDB")
	
	FriendTracker.InitDefaults()
end

function FriendTracker:InitDefaults()
    if not FriendTracker.db.factionrealm.friends then
		FriendTracker.db.factionrealm.friends={}
	end
	
	if not FriendTracker.db.factionrealm.eventQueue then
		FriendTracker.db.factionrealm.eventQueue={}
	end
    
    if not FriendTracker.db.char.frameLocation then
        FriendTracker.db.char.frameLocation={
            point="CENTER",
            relativePoint="CENTER",
            xOfs=0,
            yOfs=0,
            width=350,
            height=200,
            visible=true,
        }
    end
end

function FriendTracker:OnEnable()
	--update friends at once every 5 seconds max
	FriendTracker:RegisterBucketEvent({"FRIENDLIST_UPDATE"}, 5, "ScanFriends")	
	FriendTracker:RegisterEvent("PLAYER_LEAVING_WORLD", "OnCleanup")	
    
    FriendTracker:ShowFrame()
end

function FriendTracker:OnDisable()
end

function FriendTracker:OnCleanup()
    --save out data, before logout
    
    --save frame location
    if FriendTracker.frame then
        if FriendTracker.frame.frame:IsVisible() then
            local point, _, relativePoint, xOfs, yOfs = FriendTracker.frame.frame:GetPoint()
            FriendTracker.db.char.frameLocation = {
                point=point,
                relativePoint=relativePoint,
                xOfs=xOfs,
                yOfs=yOfs,
                width=FriendTracker.frame.frame:GetWidth(),
                height=FriendTracker.frame.frame:GetHeight(),
                visible=true,
            }
        else
            FriendTracker.db.char.frameLocation.visible=false
        end
       end
end

function FriendTracker:ResetDatabase()
	wipe(FriendTracker.db.factionrealm)
	wipe(FriendTracker.db.char)
    
    FriendTracker.AceGUI:Release(FriendTracker.frame)
    FriendTracker.frame=nil;
    
    FriendTracker:InitDefaults()
    
	FriendTracker:ScanFriends()

    FriendTracker:ShowFrame()
end

function FriendTracker:ScanFriends()
	local i=0
	
	local friend = {}
	
	for i=0,GetNumFriends() do
		local name, level, class, location, connected, status, friendNote = GetFriendInfo(i)
		
		if name ~= nil then
			friend[name]=1
		end
		
		FriendTracker:SyncFriend(name,level,class,location,connected,status,friendNote)
	end
    
    FriendTracker:RedrawFrame()
end


function FriendTracker:SyncFriend(name,level,class,location,connected,status,friendNote)

	--if the index was invalid the name might be nil
	if name then
		local info=FriendTracker.db.factionrealm.friends[name]

		local con
		
		if connected then
			con=1
		else
			con=0
		end

		if not info then
			FriendTracker.db.factionrealm.friends[name]={}
			info=FriendTracker.db.factionrealm.friends[name]
			info['level']=level
			info['class']=class
		end

		if not info['note'] and friendNote then
			info['note']=friendNote
		end

		if con == 1 then
			info['level']=level
			info['class']=class
		end
		
		if info['lastState'] ~= nil and info['lastState'] ~= con then
			-- the players connected status  changed
			table.insert(FriendTracker.db.factionrealm.eventQueue,{
				name = name,
				time = time(),
				prevtime = info['lastStateTime'],
				connected = con,
			})
		end

		info['lastStateTime']=time();
		
		info['lastState']=con
	end
end

function FriendTracker:ClearQueue()
	wipe(FriendTracker.db.factionrealm.eventQueue)
end

function FriendTracker:ShowFrame()
    if FriendTracker.frame == nil then
    
        FriendTracker.frame = FriendTracker.AceGUI:Create("Frame")
        FriendTracker.frame:SetTitle("FriendTracker")
        FriendTracker.frame:SetStatusText("")
        
        FriendTracker.frame:SetCallback("OnClose", FriendTracker.OnClose)
        FriendTracker.frame:SetCallback("OnResize", FriendTracker.OnResize)
        FriendTracker.frame:SetLayout("Fill")
        
        local scrollFrame = FriendTracker.AceGUI:Create("ScrollFrame")
        scrollFrame:SetLayout("List")
        FriendTracker.frame:AddChild(scrollFrame)
        FriendTracker.container=scrollFrame;
        
        FriendTracker.frame.statustext:GetParent():Hide()
        FriendTracker.frame.frame:SetMinResize(200,100)
        
        local info = FriendTracker.db.char.frameLocation;
        
        FriendTracker.frame.frame:SetPoint(info.point, nil, info.relativePoint, info.xOfs, info.yOfs);
        
        FriendTracker.frame.frame:SetWidth(info.width);
        FriendTracker.frame.frame:SetHeight(info.height);
        
        if not info.visible then
            FriendTracker.frame:Hide()
        end    
    else
        FriendTracker.frame:Show()
    end
    
    FriendTracker:RedrawFrame()
end

function FriendTracker:OnClose(widget)
    FriendTracker.frame:Hide()
end

function FriendTracker:RedrawFrame()
    --clean out children from the container
    FriendTracker.container:ReleaseChildren()
    
    for index, info in pairs(FriendTracker.db.factionrealm.eventQueue) do
        local group = FriendTracker.AceGUI:Create("SimpleGroup")
        group:SetLayout("Flow")
        
        group.attach=index
        
        local label = FriendTracker.AceGUI:Create("Label")
        label:SetText(string.format(
            "%s - %s", 
            info.name,
            FriendTracker:FormatTimeDelta(time()-info.time)
        ))
        label:SetFullWidth(true)
        
        if info.connected == 1 then
            label:SetColor(0,1,0)
        else
            label:SetColor(1,0,0)
        end
        
        group:AddChild(label)
        
        
        FriendTracker.container:AddChild(group)
	end 
end

function FriendTracker:PrintQueue()
	for _, info in pairs(FriendTracker.db.factionrealm.eventQueue) do
		if info['connected'] == 1 then
			FriendTracker:Printf(
				"%s was offline for %i mins has been online for %i seconds.",
				info['name'],
				floor(info['time']-info['prevtime']/60000),
				floor(time()-info['time']/1000)
			)
		else
			FriendTracker:Printf(
				"%s was online for %i mins has been offline for %i seconds.",
				info['name'],
				floor((info['time']-info['prevtime'])/60000),
				floor((time()-info['time'])/1000)
			)
		end

	end
end

function FriendTracker:FormatTimeDelta(delta)
    local day=0
    local hour=0
    local minute=0
    local second=0;
    
    --days
    while delta > 60 * 60 * 24 do
        day = day + 1
        delta = delta - 60 * 60 * 24
    end
    --hours
    while delta > 60 * 60 do
        hour = hour + 1
        delta = delta - 60 * 60
    end
    --minutes
    while delta > 60 do
        minute = minute + 1
        delta = delta - 60
    end
    --second
    second=delta
    
    local ret =''
    
    if day > 0 then
        ret = ret .. day .. 'd'
    end
    if hour > 0 then
        ret = ret .. hour .. 'h'
    end
    if minute > 0 then
        ret = ret .. minute .. 'm'
    end
    if second > 0 then
        ret = ret .. second .. 's'
    end
    
    return ret
end



































